﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using HtmlAgilityPack;
using Microsoft.EntityFrameworkCore;
using Newtonsoft.Json;
using ViraScraper.Models;
using ViraScraper.Models.JsonModels;
using ViraScraper.Repositories;
using ViraScraper.Repositories.Contracts;
using ViraScraper.Services.Contracts;
using Hashtag = ViraScraper.Models.Hashtag;

namespace ViraScraper.Services
{
    public class HashtagService : IHashtagService
    {
        private readonly IRepositoryWrapper _repository;

        public HashtagService(IRepositoryWrapper repository)
        {
            _repository = repository;
        }
        public async Task<IEnumerable<Hashtag>> GetHshtags(string hashtag)
        {
            try
            {
                List<Hashtag> returnedList = new List<Hashtag>();
                var hashtagList = _repository.Hashtag.GetAll(h => h.Tag.Contains(hashtag)).ToList();
                return hashtagList;
            }
            finally
            {
                Task.Run(() => UpdateInstagramHashtag(hashtag));
            }
        }

        public IEnumerable<Hashtag> GetHshtags(string hashtag, HashtagType type)
        {
            return null;
        }

        private void UpdateInstagramHashtag(string hash)
        {
            string uri = $"https://www.instagram.com/explore/tags/{hash}/";
            HttpClient client = new HttpClient();
            var html = client.GetStringAsync(uri).Result;
            var htmlDocument = new HtmlDocument();
            htmlDocument.LoadHtml(html);
            string postList = htmlDocument.DocumentNode.Descendants("script").Where(n => n.InnerHtml.Contains("window._sharedData")).ToList().FirstOrDefault()?.InnerHtml;
            postList = postList.Substring(20, postList.Length - 21);
            List<string> hashtagList = new List<string>();
            Dictionary<string, int> synonymses = new Dictionary<string, int>();
            var root = JsonConvert.DeserializeObject<InstaExplorerRoot>(postList);
            foreach (var tagPage in root.entry_data.TagPage)
            {
                foreach (var edge in tagPage.graphql.hashtag.edge_hashtag_to_top_posts.edges)
                {
                    foreach (var edgeC in edge.node.edge_media_to_caption.edges)
                    {
                        var hashtags = edgeC.node.text.Split('#');
                        hashtags[0] = " ";
                        hashtags[hashtags.Length - 1] = " ";
                        foreach (var hashtag in hashtags)
                        {
                            foreach (var str in hashtag.Split(' '))
                            {
                                hashtagList.AddRange(hashtag.Split('\n').Where(s => s.Contains(hash)));
                                if (synonymses.ContainsKey(str))
                                {
                                    int v = synonymses.GetValueOrDefault(str) + 1;
                                    synonymses.Remove(str);
                                    synonymses.Add(str, v);
                                }
                                else
                                    synonymses.Add(str, 0);
                            }
                        }
                    }
                }

                foreach (var hashtagToMedium in tagPage.graphql.hashtag.edge_hashtag_to_media.edges)
                {
                    foreach (var edgeC in hashtagToMedium.node.edge_media_to_caption.edges)
                    {
                        var hashtags = edgeC.node.text.Split('#');
                        hashtags[0] = " ";
                        hashtags[hashtags.Length - 1] = " ";
                        foreach (var hashtag in hashtags)
                        {
                            foreach (var str in hashtag.Split(' '))
                            {
                                hashtagList.AddRange(hashtag.Split('\n').Where(s => s.Contains(hash)));
                                if (synonymses.ContainsKey(str))
                                {
                                    int v = synonymses.GetValueOrDefault(str) + 1;
                                    synonymses.Remove(str);
                                    synonymses.Add(str, v);
                                }
                                else
                                    synonymses.Add(str, 0);
                            }
                        }
                    }
                }
            }
            hashtagList = hashtagList.Distinct().ToList();
            hashtagList.Sort((s1, s2) => { return s2.Length.CompareTo(s1.Length); });
            hashtagList.RemoveRange(0, hashtagList.Where(s => s.Length > 30).Count());
            hashtagList.ForEach(s =>
            {
                s = Regex.Replace(s, @"\t|\n|\r", "");
                s = Regex.Replace(s, @"\s+", "");
            });
            hashtagList = hashtagList.Distinct().ToList();
            List<Hashtag> hashList = new List<Hashtag>();
            hashtagList.ForEach(h =>
            {
                hashList.Add(new Hashtag
                {
                    Tag = h,
                    HashtagType = HashtagType.Instagram,
                    CreationDate = DateTime.Now
                });
            });
            _repository.Hashtag.InsertSRang(hashList);

            UpdateInstagramSynonyms(hash,synonymses);

        }

        private void UpdateInstagramSynonyms(string hash, Dictionary<string,int> synonymses)
        {
            

            var synonymsesList = synonymses.ToList();
            synonymsesList.Sort((paira,pairb)=>pairb.Value.CompareTo(paira.Value));
            synonymsesList.RemoveAll(s => s.Key == "");
            synonymsesList.RemoveAll(s => s.Key.Contains(hash));
            synonymsesList.RemoveAll(s => s.Key == " ");
            synonymsesList.RemoveAll(s => s.Key.Length<4);
        }
    }
}
